import { OpenAPIPlugin, OpenAPIPluginOptions, ParserResult } from '@opas/core';
import { ParsedOperation, WriteFileMode } from '@opas/helper';
export type PluginWriteFileMode = string | WriteFileMode | {
    api?: WriteFileMode;
    service?: WriteFileMode;
};
export interface OpenAPITransformAppPluginOptions extends OpenAPIPluginOptions {
    apiDir?: string;
    serviceDir?: string;
    dtsDir?: string;
    extractPath?: string;
    /**
     * extract field from response data
     * @example extractField = 'data'
     */
    extractField?: string | string[];
    baseUrl?: string | ((extractPath?: string) => void);
    /**
     * write file mode
     * @default 'warn'
     */
    writeFileMode?: WriteFileMode | {
        api?: WriteFileMode;
        service?: WriteFileMode;
    };
    /**
     * config param type name
     * @default AxiosRequestConfig
     */
    configParamTypeName?: string;
    /**
     * service import path
     */
    serviceImportPath?: string;
    /**
     * apis include filter
     */
    include?: ((api: ParsedOperation) => boolean) | RegExp | string[];
    /**
     * apis exclude filter
     */
    exclude?: ((api: ParsedOperation) => boolean) | RegExp | string[];
}
export default class OpenAPITransformAppPlugin extends OpenAPIPlugin<OpenAPITransformAppPluginOptions> {
    transform: ({ apis, definition, service, schema }: ParserResult) => Promise<void>;
    private outputDts;
    private joinUrl;
    private outputService;
    private outputApis;
    private createServiceImport;
    private adaptorWriteFileModeArgs;
}
