"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generator_1 = __importDefault(require("@babel/generator"));
const template_1 = __importDefault(require("@babel/template"));
const core_1 = require("@opas/core");
const helper_1 = require("@opas/helper");
const fs_extra_1 = __importDefault(require("fs-extra"));
const lodash_1 = require("lodash");
const node_os_1 = __importDefault(require("node:os"));
const node_path_1 = __importDefault(require("node:path"));
const constants_1 = require("./constants");
class OpenAPITransformAppPlugin extends core_1.OpenAPIPlugin {
    constructor() {
        super(...arguments);
        this.transform = (_a) => __awaiter(this, [_a], void 0, function* ({ apis, definition, service, schema }) {
            var _b, _c;
            const { writeFileMode, include, exclude } = this.options;
            const { api: apiMode = 'skip', service: serviceMode = 'warn' } = this.adaptorWriteFileModeArgs(writeFileMode);
            const filteredApis = apis
                .filter((api) => {
                if (include) {
                    if (typeof include === 'function') {
                        return include(api);
                    }
                    else if (Array.isArray(include)) {
                        return include.includes(api.operationId);
                    }
                    else {
                        return include.test(api.uri);
                    }
                }
                else {
                    return true;
                }
            })
                .filter((api) => {
                if (exclude) {
                    if (typeof exclude === 'function') {
                        return !exclude(api);
                    }
                    else if (Array.isArray(exclude)) {
                        return !exclude.includes(api.operationId);
                    }
                    else {
                        return !exclude.test(api.uri);
                    }
                }
                else {
                    return true;
                }
            });
            yield this.outputApis({
                apis: filteredApis,
                service,
                writeFileMode: apiMode,
                openApiVersion: (_b = schema.openApiVersion) !== null && _b !== void 0 ? _b : 2,
                configParamTypeName: (_c = this.options.configParamTypeName) !== null && _c !== void 0 ? _c : constants_1.DEFAULT_REQUEST_CONFIG_PARAM_TYPE_NAME,
                schema,
            });
            yield this.outputService(service.basePath, service, serviceMode);
            yield this.outputDts(service, definition);
        });
        this.outputDts = (service, definition) => __awaiter(this, void 0, void 0, function* () {
            const { cwd = process.cwd(), dtsDir = node_path_1.default.join(cwd, 'src/interfaces') } = this.options;
            const dtsFile = node_path_1.default.join(dtsDir, `${(0, lodash_1.kebabCase)(service.name)}.d.ts`);
            const formattedContent = yield (0, helper_1.formatCode)({
                source: definition,
                filePath: dtsFile,
            });
            yield (0, helper_1.outFile)({
                tips: {
                    start: 'start generate dts file',
                    error: 'dts file generate failed',
                    success: `dts file generate success: ${dtsFile}`,
                },
                code: formattedContent,
                outFileName: dtsFile,
                writeFileMode: 'overwrite',
            });
        });
        this.joinUrl = (pre = '', next = '') => {
            if (next.startsWith('http')) {
                return next;
            }
            else {
                return pre.replace(/\/$/, '') + '/' + next.replace(/^\//, '');
            }
        };
        this.outputService = (...args_1) => __awaiter(this, [...args_1], void 0, function* (basePath = '', service, writeFileMode) {
            const { cwd = process.cwd(), serviceDir = node_path_1.default.join(cwd, 'src/services'), extractPath = '', baseUrl, serviceImportPath = '../utils/service', } = this.options;
            const serviceName = service.name;
            const mergedPath = this.joinUrl(basePath, extractPath);
            const templateSource = yield fs_extra_1.default.readFile(node_path_1.default.join(__dirname, '../templates/service.tpl'), {
                encoding: 'utf-8',
            });
            const templateAst = (0, template_1.default)(templateSource, {
                plugins: ['typescript'],
            });
            const statements = templateAst({
                SERVICE_NAME: `${serviceName}Service`,
                BASE_URL: typeof baseUrl === 'function' ? baseUrl(mergedPath) : `'${this.joinUrl(baseUrl, mergedPath)}'`,
                SERVICE_PATH: serviceImportPath,
            });
            const code = statements.map((statement) => (0, generator_1.default)(statement).code).join('\n');
            const outFileName = serviceDir + `/${(0, lodash_1.kebabCase)(serviceName)}.ts`;
            const formattedContent = yield (0, helper_1.formatCode)({
                source: code,
                filePath: outFileName,
                parser: 'typescript',
            });
            yield (0, helper_1.outFile)({
                tips: {
                    start: 'start generate service file',
                    error: 'service file generate failed',
                    success: `service file generate success: ${outFileName}`,
                },
                code: formattedContent,
                outFileName,
                writeFileMode,
            });
        });
        this.outputApis = (_a) => __awaiter(this, [_a], void 0, function* ({ apis, service, writeFileMode, openApiVersion, configParamTypeName = constants_1.DEFAULT_REQUEST_CONFIG_PARAM_TYPE_NAME, schema, }) {
            const { cwd = process.cwd(), apiDir = node_path_1.default.join(cwd, 'src/apis'), extractField } = this.options;
            const serviceName = service.name;
            const apiAstTemplate = yield fs_extra_1.default.readFile(node_path_1.default.join(__dirname, '../templates/api.tpl'), {
                encoding: 'utf-8',
            });
            const render = (0, helper_1.createRenderer)(apiAstTemplate, {
                openApiVersion,
                extractField,
                configParamTypeName,
                schema,
            });
            // add service import code
            const importServiceCode = this.createServiceImport(service);
            // add apis
            const apisCode = apis
                .map((api) => {
                return render(api, serviceName);
            })
                .join(node_os_1.default.EOL);
            const serviceFileName = (0, lodash_1.kebabCase)(serviceName);
            const outFileName = node_path_1.default.join(apiDir, `${serviceFileName}.ts`);
            const content = `
    import type { ${configParamTypeName} } from 'axios';

    ${importServiceCode}
    ${apisCode}
    `;
            const formattedContent = yield (0, helper_1.formatCode)({
                source: content,
                filePath: outFileName,
                parser: 'typescript',
            });
            yield (0, helper_1.outFile)({
                tips: {
                    start: 'start generate api file',
                    error: 'api file generate failed',
                    success: `api file generate success: ${outFileName}`,
                },
                outFileName,
                code: formattedContent,
                writeFileMode,
            });
        });
        this.createServiceImport = (service) => {
            const serviceName = service.name;
            const importee = (0, helper_1.formatService)(serviceName);
            const serviceFileName = (0, lodash_1.kebabCase)(serviceName);
            const importer = node_path_1.default.posix.join(constants_1.DEFAULT_SERVICE_DIR, serviceFileName);
            return `import ${importee} from '${importer}';`;
        };
        this.adaptorWriteFileModeArgs = (writeFileMode = 'warn') => {
            if (typeof writeFileMode === 'string') {
                return {
                    api: writeFileMode,
                    service: writeFileMode,
                };
            }
            return writeFileMode;
        };
    }
}
exports.default = OpenAPITransformAppPlugin;
